/*
** hexa.c for elfsh
** 
** Started on  Thu Feb 28 16:33:57 2002 mayhem
** Last update Sat Mar  2 19:22:25 2002 mayhem
*/
#include "elfsh.h"




/* Print data in hexadecimal and ascii */
int             print_hexa(Elfsh32_Obj *file, char *name, u_int addr, int len, u_int type)
{
  Elfsh32_Sect	*sect;
  char		*buff;
  int		index;
  int		value;
  char		*base = "0123456789ABCDEF";
  char		off[256];
  int		offset;

  /* If the object is a section, its size is not in its symbol entry but in the sh_size field in the section header table */
  if (type == STT_SECTION)
    {
      sect = elfsh_get_parent_section(file, addr, NULL);
      if (!sect)
	{
	  printf(" => \t Object has an invalid virtual address . \n");
	  return (0);
	}
      len = sect->shdr->sh_size;
    }

  /* Get the data */
  XALLOC(buff, len, 0);
  len = elfsh_raw_read(file, addr, buff, len);
  if (!len)
    {
      printf(" => \t Object has its size field set to 0 . \n");
      return (0);
    }

  /* If the symbol is just a tiny variable */
  else if (type != STT_FUNC && type != STT_SECTION && len <= 4)
    {
      value = *(int *) buff;
      printf(" => \t Hexadecimal(%X) Decimal(%u) Octal(%o) \n", value, value, value);
      return (len);
    }

  
  putchar('\n');


  /* Print in hexadecimal doing reverse symbol matching */
  for (index = 0; index < len; index += 8)
    {
      
      name = elfsh_reverse_symbol(file, addr + index, &offset);
      if (!name)
	name = ELFSH_NULL_STRING;
      if (offset)
	snprintf(off, sizeof(off), " + %u", offset);
      else
	snprintf(off, sizeof(off), "\t");

      printf("\n %40s%s \t%s %10p : \t\t %c%c %c%c %c%c %c%c  %c%c %c%c %c%c %c%c \t %c%c%c%c%c%c%c%c",
	     name, off, (offset > 0 && offset < 100 ? "\t" : ""), (void *) addr + index,
	     (index + 0 >= len ? ' ' : base[(buff[index] >> 4) & 0x0F]),
	     (index + 0 >= len ? ' ' : base[buff[index] & 0x0F]),
	     (index + 1 >= len ? ' ' : base[(buff[index + 1] >> 4) & 0x0F]),
	     (index + 1 >= len ? ' ' : base[buff[index + 1] & 0x0F]),
	     (index + 2 >= len ? ' ' : base[(buff[index + 2] >> 4) & 0x0F]),
	     (index + 2 >= len ? ' ' : base[buff[index + 2] & 0x0F]),
	     (index + 3 >= len ? ' ' : base[(buff[index + 3] >> 4) & 0x0F]),
	     (index + 3 >= len ? ' ' : base[buff[index + 3] & 0x0F]),
	     (index + 4 >= len ? ' ' : base[(buff[index + 4] >> 4) & 0x0F]),
	     (index + 4 >= len ? ' ' : base[buff[index + 4] & 0x0F]),
	     (index + 5 >= len ? ' ' : base[(buff[index + 5] >> 4) & 0x0F]),
	     (index + 5 >= len ? ' ' : base[buff[index + 5] & 0x0F]),
	     (index + 6 >= len ? ' ' : base[(buff[index + 6] >> 4) & 0x0F]),
	     (index + 6 >= len ? ' ' : base[buff[index + 6] & 0x0F]),
	     (index + 7 >= len ? ' ' : base[(buff[index + 7] >> 4) & 0x0F]),
	     (index + 7 >= len ? ' ' : base[buff[index + 7] & 0x0F]),
	     (index + 0 >= len ? ' ' : PRINTABLE(buff[index + 0]) ? buff[index + 0] : '.'),
	     (index + 1 >= len ? ' ' : PRINTABLE(buff[index + 1]) ? buff[index + 1] : '.'),
	     (index + 2 >= len ? ' ' : PRINTABLE(buff[index + 2]) ? buff[index + 2] : '.'),
	     (index + 3 >= len ? ' ' : PRINTABLE(buff[index + 3]) ? buff[index + 3] : '.'),
	     (index + 4 >= len ? ' ' : PRINTABLE(buff[index + 4]) ? buff[index + 4] : '.'),
	     (index + 5 >= len ? ' ' : PRINTABLE(buff[index + 5]) ? buff[index + 5] : '.'),
	     (index + 6 >= len ? ' ' : PRINTABLE(buff[index + 6]) ? buff[index + 6] : '.'),
	     (index + 7 >= len ? ' ' : PRINTABLE(buff[index + 7]) ? buff[index + 7] : '.'));

    }

      puts("\n");

  free(buff);
  return (0);

}





/* Display file objects in hexadecimal */
int             display_hexa(Elfsh32_Obj *file, argv_t *av)
{
  Elf32_Sym	*symtab;
  list_t	*actual;
  int		num;
  int		index;
  char		*name;
  int		type;

  symtab = elfsh_get_symtab(file, &num);
  if (!symtab)
    RET(-1);

  puts("\n[HEXADECIMAL VIEW]\n");

  for (actual = av->hexa; actual; actual = actual->next)
    for (index = 0; index < num; index++)
      {

	name = elfsh_get_symbol_name(file, symtab + index);
	if (name == NULL)
	  name = ELFSH_NULL_STRING;

	if (!regexec(&actual->name, name, 0, 0, 0))
	  {
	    
	    type = elfsh_get_symbol_type(symtab + index);

	    if (type == STT_FUNC || type == STT_SECTION || type == STT_COMMON || type == STT_OBJECT)
	      {
		printf("\n[############ %s %30s at 0x%08X ##############]", 
		       (type == STT_FUNC ? "Function" : type == STT_SECTION ? "Section" : "Variable"), 
		       name, symtab[index].st_value);
		
		fflush(stdout);

		print_hexa(file, name, symtab[index].st_value, symtab[index].st_size, type);
	      }

	  }
      }
  return (-1);
}




















