/*
** libelfsh.h for libelfsh
** 
** Started on  Mon Jul 23 15:47:12 2001 mayhem
** Last update Fri Mar  1 01:21:05 2002 mayhem
*/
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>



/* Portability Stuffs */

#if defined(__OpenBSD__) || defined(__NetBSD__)
 #include <sys/exec_elf.h>
#elif defined(__linux__) || defined(__FreeBSD__) || defined(sun)
 #include <elf.h>
#endif 

#ifndef DT_BIND_NOW		/* undefined on BSD */
 #define DT_BIND_NOW 24
#endif

#if defined(sun)		/* SunOS is high sometimes ... */
 #include <strings.h>
 #include "libelfsh-sunos.h"
#endif

#if defined(sun)
 #define		ELFSH_ERR_ARRAY		strerror(errno)
#else
 #define		ELFSH_ERR_ARRAY		sys_errlist[sys_nerr]
#endif


/* Safe calloc() */
#define		XALLOC(a, b, c)				\
do							\
{							\
  if ((a = calloc(b, 1)) == NULL)			\
    ELFSH_SETERROR("libelfsh: Out of memory .", c);	\
}							\
while (0)

/* Safe open() */
#define		XOPEN(a, b, c, d, e)	\
do					\
{					\
  if ((a = open(b, c, d)) < 0)		\
    return (e);				\
}					\
while (0)
     

/* Safe read() */
#define		XREAD(a, b, c, d)		\
do						\
{						\
  if (read(a, b, c) != c)			\
    ELFSH_SETERROR(ELFSH_ERR_ARRAY, d);		\
}						\
while (0)


/* Safe lseek() */
#define		XSEEK(a, b, c, d)		\
do						\
{						\
  if (lseek(a, b, c) == (int) b - 1)	        \
    ELFSH_SETERROR(ELFSH_ERR_ARRAY, d);		\
}						\
while (0)



/* Simple useful boolean macro */
#define		INTERVAL(a, b, c) (a <= b && b < c)



/*
** Some error stuffs : set the error offset of the file when calling SETERROR
** a: the file structure
** b: the string error
** c: the return value of the function
*/
#define ELFSH_SETERROR(b, c)		\
do					\
{					\
  elfsh_error_msg = (char *) b;		\
  return (c);				\
}					\
while (0)			




/* Some defined values */
#define		ELFSH_SECTION_NAME_PLT		".plt"
#define		ELFSH_SECTION_NAME_DTORS	".dtors"
#define		ELFSH_SECTION_NAME_CTORS	".ctors"
#define		ELFSH_SECTION_NAME_GOT		".got"
#define		ELFSH_SECTION_NAME_INTERP	".interp"

#define		ELFSH_PLT_ENTRY_SIZE	        16
#define		ELFSH_SYMTAB_ENTRY_SIZE		sizeof(Elf32_Sym)

#define		ELFSH_NULL_STRING		"(NULL)"
#define		ELFSH_SORT_BY_ADDR		'a'
#define		ELFSH_SORT_BY_SIZE		's'
#define		ELFSH_RULEZ_THE_WORLD		42

#define		ELFSH_SECTION_FIRST		0
#define		ELFSH_SECTION_LAST		0xFFFF
#define		ELFSH_FUNCTION_FIRST		0
#define		ELFSH_FUNCTION_LAST		0xFFFF




/* Extern variables */
extern char	*elfsh_error_msg;







/* --------------------- structures ------------------ */



/* Stab entry */
typedef struct		s_stab
{
  unsigned long		strindex;
  unsigned char		type;
  unsigned char		other;
  unsigned short	desc;
  unsigned int		value;
}			Elfsh32_Stab_ent;





/* Note section entry */
typedef struct		s_nentry
{
  int			namesz;
  int			descsz;
  int			type;
  char			*note;
  char			*desc;
  struct s_nentry	*next;
  struct s_nentry	*prev;
}			Elfsh32_Note_ent;


/* PLT entry */
typedef struct		s_pltent
{
  u_char		data[ELFSH_PLT_ENTRY_SIZE];
}			Elfsh32_Plt_ent;



/* forward declaration */
struct			s_sect;	


/* Function type */
typedef struct		s_func
{
  Elf32_Sym		*sym;
  struct s_sect		*section;	/* section the function belongs to	*/
  int			offset;		/* Section offset for this function	*/
  int			len;		/* function real lenght			*/
  struct s_func		*next;
}			Elfsh32_Func;



/* forward declaration */
struct s_obj;


/* Section data type */
typedef struct		s_sect
{

  /* Filled at sht loading */
  Elf32_Shdr		*shdr;		/* section header for this section	*/
  Elf32_Phdr		*phdr;		/* Program header for this section	*/
  int			index;		/* Section index in sht			*/
  struct s_sect		*next;		/* next section in the list		*/
  struct s_sect		*prev;		/* prev section in the list		*/
  struct s_obj		*parent;	/* parent ELF object file		*/	

  /* Filled at section loading */
  void			*data;		/* section's data (type dependant)	*/
  int			nbr;		/* nbr of elements in the data field	*/

  /* Alternative field */
  void			*altdata;	/* SH_SYMTAB stores a sorted table here, +x section stores the fht */
  int			altnbr;		/* Number of units for altdata		*/

  /* Filled at fingerprint time (not yet used) */
  /* char		*name;		section name !				*/
  /* int		type;		ELFsh section fingerprint		*/


}			Elfsh32_Sect;




#define		ELFSH_SECTION_NULL		0
#define		ELFSH_SECTION_INTERP		1
#define		ELFSH_SECTION_NOTE		2
#define		ELFSH_SECTION_HASH		3
#define		ELFSH_SECTION_DYNSYM		4
#define		ELFSH_SECTION_DYNSTR		5
#define		ELFSH_SECTION_GNUVER		6
#define		ELFSH_SECTION_RELGOT		7
#define		ELFSH_SECTION_RELBSS		8
#define		ELFSH_SECTION_RELPLT		9
#define		ELFSH_SECTION_INIT		10
#define		ELFSH_SECTION_PLT		11
#define		ELFSH_SECTION_TEXT		12
#define		ELFSH_SECTION_FINI		13
#define		ELFSH_SECTION_RODATA		14
#define		ELFSH_SECTION_DATA		15
#define		ELFSH_SECTION_EHFRAME		16
#define		ELFSH_SECTION_CTORS		17
#define		ELFSH_SECTION_DTORS		18
#define		ELFSH_SECTION_GOT		19
#define		ELFSH_SECTION_DYNAMIC		20
#define		ELFSH_SECTION_SBSS		21
#define		ELFSH_SECTION_STAB		22
#define		ELFSH_SECTION_COMMENT		23
#define		ELFSH_SECTION_SHSTRTAB		24
#define		ELFSH_SECTION_SYMTAB		25
#define		ELFSH_SECTION_STRTAB		26
#define		ELFSH_SECTION_UNKNOWN		255






/* ELF object structure */
typedef struct	s_obj
{
  
  /* File descriptor for the original file */
  int		fd;

  /* Object path */
  char		*name;

  /* ELFSH_OBJECT_CORE, ELFSH_OBJECT_SHARED, ELFSH_OBJECT_RELOC or ELFSH_OBJECT_EXEC */
  int		type;	

  /* Elf headers */
  Elf32_Ehdr	*hdr;
  Elf32_Shdr	*sht;
  Elf32_Phdr	*pht;

  /* Section linked list */
  Elfsh32_Sect	*sections;


  /* ********************************************************************************** */
  /* The following variables point to their respective entry in the section linked list */
  /* ********************************************************************************** */

  /* Symbol table related fields */
  Elfsh32_Sect	*symtab;

  /* Dynamic symbol table related fields */
  Elfsh32_Sect	*dynsym;

  /* Stab section fields (debugging symbols) */
  Elfsh32_Sect	*stab;

  /* Symbol Hash (.hash) table fields */
  Elfsh32_Sect	*hash;

  /* Dynamic structure fields (PT_DYNAMIC) */
  Elfsh32_Sect	*dynamic;

  /* Constructors */
  Elfsh32_Sect	*ctors;

  /* Destructors */
  Elfsh32_Sect	*dtors;

  /* Global Offset Table fields */
  Elfsh32_Sect	*got;
  
  /* Procedure Linkage Table (.plt) object */
  Elfsh32_Sect	*plt;

  /* Interpretor section (PT_INTERP) */
  Elfsh32_Sect	*interp;

  /* Symbol string table (.strtab) */
  Elfsh32_Sect	*strtab;

  /* Stab string table (.stabstr) */
  Elfsh32_Sect	*stab_strtab;
  
  /* Dynamic symbol string table (.dynstr) */
  Elfsh32_Sect	*dyn_strtab;

  /* Section string table (.shstrtab) */
  Elfsh32_Sect	*sh_strtab;

  /* Various internal fields for error handling */
  char		buff[ELFSH_RULEZ_THE_WORLD];
  const char	*error;

  /* Has the object already been function scanned ? */
  char		scanned;		

  /* The list is double linked */
  void		*next;
  void		*prev;

}		Elfsh32_Obj;






/* ----------------- prototypes -------------------------- */






/* libelfsh API, check api.txt for a complete description */
Elfsh32_Obj	*elfsh_load_obj(char *name);

Elfsh32_Sect	*elfsh_get_section_by_type(Elfsh32_Obj *, u_int, int *, int *, int *, int);
Elfsh32_Sect	*elfsh_get_section_by_index(Elfsh32_Obj *, int, int *, int *);
Elfsh32_Sect	*elfsh_get_section_by_name(Elfsh32_Obj *, char *, int *, int *, int *);
Elfsh32_Sect	*elfsh_get_parent_section(Elfsh32_Obj *file, u_int value, int *offset);
Elfsh32_Sect	*elfsh_get_tht(Elfsh32_Obj *file);
Elfsh32_Sect	*elfsh_get_plt(Elfsh32_Obj *file, int *num);
Elfsh32_Sect	*elfsh_get_notes(Elfsh32_Obj *file, u_int range, char *name);
Elfsh32_Sect	*elfsh_get_reloc(Elfsh32_Obj *file, u_int range, char *name, u_int *num);

Elfsh32_Func	*elfsh_get_function_by_addr(Elfsh32_Obj *file, int addr);
Elfsh32_Func    *elfsh_get_function_by_name(Elfsh32_Obj *file, char *name);

Elf32_Phdr	*elfsh_get_segment_by_type(Elfsh32_Obj *file, int type, int range);
Elf32_Phdr	*elfsh_get_parent_pheader(Elfsh32_Sect *new);

Elf32_Shdr      *elfsh_get_section_header_from_sym(Elfsh32_Obj *file, Elf32_Sym *sym);

Elf32_Dyn	*elfsh_get_dynamic(Elfsh32_Obj *file, int *num);

Elf32_Sym	*elfsh_get_symbol_by_name(Elfsh32_Obj *file, char *name);
Elf32_Sym	*elfsh_copy_symtab(void *addr, int size);
Elf32_Sym	*elfsh_merge_symtabs(Elf32_Sym *one, Elf32_Sym *two, int size_one, int size_two);
Elf32_Sym	*elfsh_get_symbol_from_reloc(Elfsh32_Obj *file, Elf32_Rel *s);

void		*elfsh_get_hdr(Elfsh32_Obj *file);
void		*elfsh_get_stab(Elfsh32_Obj *file, int *num);
void		*elfsh_get_dynsymtab(Elfsh32_Obj *file, int *num);
void		*elfsh_get_symtab(Elfsh32_Obj *file, int *num);
void		*elfsh_get_sht(Elfsh32_Obj *file, int *num);
void		*elfsh_get_pht(Elfsh32_Obj *file, int *num);
void		*elfsh_get_hashtable(Elfsh32_Obj *file);
void		*elfsh_get_got(Elfsh32_Obj *file, int *num);
void		*elfsh_get_dtors(Elfsh32_Obj *file, int *num);
void		*elfsh_get_ctors(Elfsh32_Obj *file, int *num);
void		*elfsh_get_stab_offset(Elfsh32_Stab_ent *s);	       
void		*elfsh_load_section(Elfsh32_Obj *file, Elf32_Shdr *shdr);
void		*elfsh_get_reloffset(Elfsh32_Obj *file, Elf32_Rel *r);
void		*elfsh_get_symbol_offset(Elf32_Sym *s);
void		*elfsh_get_anonymous_section(Elfsh32_Obj *file, Elfsh32_Sect *sect);

char		*elfsh_reverse_symbol(Elfsh32_Obj *file, u_int sym_value, int *offset);
char		*elfsh_reverse_dynsymbol(Elfsh32_Obj *file, u_int sym_value, int *offset);
char		*elfsh_get_interp(Elfsh32_Obj *file);
char		*elfsh_get_dynentry_info(Elfsh32_Obj *file, Elf32_Dyn *d);
char		*elfsh_get_stab_name(Elfsh32_Obj *file, Elfsh32_Stab_ent *s);
char		*elfsh_get_dynsymbol_name(Elfsh32_Obj *file, Elf32_Sym *s);
char		*elfsh_get_symbol_name(Elfsh32_Obj *file, Elf32_Sym *s);
char		*elfsh_get_section_name(Elfsh32_Obj *file, Elf32_Shdr *s);

u_int		elfsh_get_symbol_bind(Elf32_Sym *s);
u_int		elfsh_get_symbol_type(Elf32_Sym *s);
u_int		elfsh_get_segment_type(Elf32_Phdr *p);
u_int		elfsh_get_section_type(Elf32_Shdr *s);
u_int		elfsh_get_objtype(Elf32_Ehdr *e);
u_int		elfsh_get_arch(Elf32_Ehdr *e);
u_int		elfsh_get_reloc_type(Elf32_Rel *r);
u_int		elfsh_get_dynentry_type(Elf32_Dyn *d);
u_int		elfsh_get_stab_type(Elfsh32_Stab_ent *s);

int		elfsh_get_section_align(Elf32_Shdr *s);
int		elfsh_get_section_vaddr(Elf32_Shdr *s);
int		elfsh_get_section_foffset(Elf32_Shdr *s);

int		elfsh_get_symbol_hash(char *sym_name);
int		elfsh_get_symbol_value(Elfsh32_Obj *file, char *sym_name);
int		elfsh_load_pht(Elfsh32_Obj *file);
int		elfsh_load_hdr(Elfsh32_Obj *file);
int		elfsh_load_sht(Elfsh32_Obj *file);
int		elfsh_section_is_executable(Elf32_Shdr *s);
int		elfsh_section_is_writable(Elf32_Shdr *s);
int		elfsh_section_is_allocatable(Elf32_Shdr *s);
int		elfsh_get_section_size(Elf32_Shdr *s);
int		elfsh_segment_is_writable(Elf32_Phdr *p);
int		elfsh_segment_is_readable(Elf32_Phdr *p);
int		elfsh_segment_is_executable(Elf32_Phdr *p);
int		elfsh_get_symbol_size(Elf32_Sym *s);
int		elfsh_is_pltentry(Elfsh32_Obj *file, Elf32_Sym *sym);
int		elfsh_get_function_len(Elfsh32_Func *func);
int		elfsh_get_function_addr(Elfsh32_Func *func);
int		elfsh_is_plt(Elfsh32_Obj *file, Elf32_Shdr *s);
int		elfsh_insert_tht_entry(Elfsh32_Obj *file, Elfsh32_Sect *sct, int range);
int		elfsh_insert_fht_entry(Elfsh32_Sect *sct, Elfsh32_Func *fct, int range);
int		elfsh_sort_symtab(Elf32_Sym *symtab, int size, int type);
int		elfsh_fingerprint_section(Elfsh32_Sect *sect);
int		elfsh_scan_functions(Elfsh32_Obj *file);
int		elfsh_sort_functions(Elfsh32_Obj *file);
int		elfsh_raw_write(Elfsh32_Obj *file, u_int dst, void *src, int len);
int		elfsh_raw_read(Elfsh32_Obj *file, u_int dst, void *src, int len);

void		elfsh_unload_obj(Elfsh32_Obj *file);
void		elfsh_print_error(Elfsh32_Obj *file);
void		elfsh_free_reloc_list(Elfsh32_Obj *file);
void		elfsh_free_notes_list(Elfsh32_Obj *file);
void		elfsh_free_tht(Elfsh32_Obj *file);
void		elfsh_error();
void		elfsh_add_section(Elfsh32_Obj *file, Elfsh32_Sect *sect, int index);











