/*
** notes.c for libelfsh
** 
** Started on  Sun Jun 10 22:57:09 2001 mayhem
** Last update Tue Feb 26 21:50:44 2002 mayhem
*/
#include "libelfsh.h"





/* Return the content of the 'range'th section of type SHT_NOTE */
Elfsh32_Sect		*elfsh_get_notes(Elfsh32_Obj *file, u_int range, char *name)
{
  Elfsh32_Sect		*notes;
  Elfsh32_Note_ent	*e;
  Elfsh32_Note_ent	*tmp;	
  int			offset;
  int			size;
  char			*cache;

  notes = elfsh_get_section_by_type(file, SHT_NOTE, NULL, NULL, &size, range);
  if (!notes)
    return (NULL);

  notes->nbr = size / sizeof(Elfsh32_Note_ent);

  if ((cache = elfsh_load_section(file, notes->shdr)) == NULL)
    return (NULL);
  
  /* Parse all the note entries of the section */
  for (offset = 0; offset < notes->shdr->sh_size; offset += (e->namesz + e->descsz + sizeof(int) * 3))
    {
      
      XALLOC(e, sizeof(Elfsh32_Note_ent), NULL);
      
      e->namesz    = * (int *) cache;
      e->namesz   += e->namesz % 4;
      e->descsz    = * (int *) cache + 1;
      e->descsz   += e->descsz % 4;
      e->note      = strdup((char *) cache + offset + sizeof(int) * 3);
      e->desc      = strdup((char *) cache + offset + sizeof(int) * 3 + e->namesz);
      
      /* Put the note entry at the end of the list */
      if (!notes->data)
	notes->data = e;
      else
	{
	  tmp = notes->data;
	  while (tmp->next)
	    tmp = tmp->next;
	  tmp->next = e;
	  e->prev = tmp;
	}
      
    }
  
  free(cache);
  return (notes);
  
}






/* Free the notes section linked list */
void			elfsh_free_notes(Elfsh32_Sect *sect)
{
  Elfsh32_Note_ent	*e;
  Elfsh32_Note_ent	*etmp;
  
  for (e = sect->data; e; e = etmp)
    {
      etmp = e->next;
      free(e);
    }
}









